@@ -1,7 +1,6 @@
 
 package Apache2::ASP;
 
-our $VERSION = 0.18;
 
 use strict;
 use warnings 'all';
@@ -17,8 +16,9 @@ use Apache2::Directive ();
 use Apache2::Connection ();
 use Apache2::SubRequest ();
 use Apache2::RequestUtil ();
-use Devel::StackTrace;
 
+use vars '$VERSION';
+$VERSION = '1.56_1';
 
 #==============================================================================
 sub handler : method
@@ -27,28 +27,38 @@ sub handler : method
   
   # We function best as an object:
   my $s = $class->SUPER::new( $ENV{APACHE2_ASP_CONFIG} );
+  $main::_ASP::ASP = $s;
+  $s->{r} = $r;
   
   # What Apache2::ASP::Handler is going to handle this request?
-  my $handler_class = $s->resolve_request_handler( $r->uri );
-  if( $handler_class->isa('Apache2::ASP::UploadHandler') )
+  if( uc($ENV{REQUEST_METHOD}) eq 'POST'  )
   {
-    # We use the upload_hook functionality from Apache::Request
-    # to process uploads:
-    my $hook_obj = Apache2::ASP::UploadHook->new(
-      asp           => $s,
-      handler_class => $handler_class,
-    );
-    $s->{q} = Apache2::ASP::CGI->new( $r, sub { $hook_obj->hook( @_ ) } );
+    my $handler_class = $s->resolve_request_handler( $r->uri );
+    if( $handler_class->isa('Apache2::ASP::UploadHandler') )
+    {
+      # We use the upload_hook functionality from Apache::Request
+      # to process uploads:
+      my $hook_obj = Apache2::ASP::UploadHook->new(
+        asp           => $s,
+        handler_class => $handler_class,
+      );
+      $s->{'q'} = Apache2::ASP::CGI->new( $r, sub { $hook_obj->hook( @_ ) } );
+    }
+    else
+    {
+      # Not an upload - normal CGI functionality will work fine:
+      $s->{'q'} = Apache2::ASP::CGI->new( $r );
+    }# end if()
   }
   else
   {
     # Not an upload - normal CGI functionality will work fine:
-    $s->{q} = Apache2::ASP::CGI->new( $r );
+    $s->{'q'} = Apache2::ASP::CGI->new( $r );
   }# end if()
   
   # Get our subref and execute it:
-  my $handler = $s->setup_request( $r, $s->{q} );
-  my $status = eval { $handler->( ) };
+  $s->setup_request( $r, $s->{'q'} );
+  my $status = eval { $s->execute() };
   if( $@ )
   {
     warn "ERROR AFTER CALLING \$handler->( ): $@";
@@ -56,7 +66,7 @@ sub handler : method
   }# end if()
   
   # 0 = OK, everything else means errors of some kind:
-  return $status;
+  return $status eq '200' ? 0 : $status;
 }# end handler()
 
 
@@ -65,10 +75,9 @@ sub _handle_error
 {
   my ($s, $err) = @_;
   
-  my $stack = Devel::StackTrace->new;
-  warn "$@ -- " . $stack->as_string;
+  warn $err;
   $s->response->Clear();
-  $s->global_asa->can('Script_OnError')->( $stack );
+  $s->global_asa->can('Script_OnError')->( $err );
   
   return 500;
 }# end _handle_error()
@@ -127,7 +136,7 @@ Use RT L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Apache2-ASP> to submit bug rep
 
 =head1 HOMEPAGE
 
-Please visit the Apache2::ASP homepage at L<http://apache2-asp.no-ip.org/> to see examples
+Please visit the Apache2::ASP homepage at L<http://www.devstack.com/> to see examples
 of Apache2::ASP in action.
 
 =head1 AUTHOR
